/*
THIS INFRAGISTICS ULTIMATE SOFTWARE LICENSE  AGREEMENT ("AGREEMENT") LOCATED HERE:
https://www.infragistics.com/legal/license/igultimate-la
https://www.infragistics.com/legal/license/igultimate-eula
GOVERNS THE LICENSING, INSTALLATION AND USE OF INFRAGISTICS SOFTWARE. BY DOWNLOADING AND/OR INSTALLING AND USING INFRAGISTICS SOFTWARE:  you are indicating that you have read and understand this Agreement, and agree to be legally bound by it on behalf of the yourself and your company.
*/
import { Base, markType } from "./type";
import { Random } from "./Random";
import { FinancialDataPosition } from "./FinancialDataPosition";
import { FinancialDataSource } from "./FinancialDataSource";
import { FinancialDataItem } from "./FinancialDataItem";
import { FinancialDataLookup } from "./FinancialDataLookup";
import { truncate } from "./number";
import { dateNow, dateFromValues, dateGetMonth, dateAddMonths, dateAddHours, dateGetTimeOfDay } from "./date";
import { timeSpanTotalHours, timeSpanInit1 } from "./timespan";
import { stringIsNullOrEmpty } from "./string";
import { round10N } from "./numberExtended";
/**
 * @hidden
 */
export class FinancialDataGenerator extends Base {
    constructor() {
        super();
        this._priceStart = 0;
        this._priceRange = 0;
        this._priceMinimum = 0;
        this._priceMaximum = 0;
        this._volumeStart = 0;
        this._volumeRange = 0;
        this._dateRange = 0;
        this._dateInterval = 0;
        this._stockInterval = 0;
        this._useAfterHours = false;
        this.f = new Date();
        this.g = new Date();
        this.p = new Random(0);
        this.useAfterHours = true;
        this.priceStart = 0;
        this.priceRange = NaN;
        this.priceMinimum = NaN;
        this.priceMaximum = NaN;
        this.volumeStart = 0;
        this.volumeRange = NaN;
        this.dateRange = 12;
        this.dateInterval = 24;
    }
    get priceStart() {
        return this._priceStart;
    }
    set priceStart(a) {
        this._priceStart = a;
    }
    get priceRange() {
        return this._priceRange;
    }
    set priceRange(a) {
        this._priceRange = a;
    }
    get priceMinimum() {
        return this._priceMinimum;
    }
    set priceMinimum(a) {
        this._priceMinimum = a;
    }
    get priceMaximum() {
        return this._priceMaximum;
    }
    set priceMaximum(a) {
        this._priceMaximum = a;
    }
    get volumeStart() {
        return this._volumeStart;
    }
    set volumeStart(a) {
        this._volumeStart = a;
    }
    get volumeRange() {
        return this._volumeRange;
    }
    set volumeRange(a) {
        this._volumeRange = a;
    }
    get dateRange() {
        return this._dateRange;
    }
    set dateRange(a) {
        this._dateRange = a;
    }
    get dateInterval() {
        return this._dateInterval;
    }
    set dateInterval(a) {
        this._dateInterval = a;
    }
    get stockInterval() {
        return this._stockInterval;
    }
    set stockInterval(a) {
        this._stockInterval = a;
    }
    get useAfterHours() {
        return this._useAfterHours;
    }
    set useAfterHours(a) {
        this._useAfterHours = a;
    }
    generatePosition(a, b = "") {
        let c = this.generateData(a, b);
        let d = this.p.next2(truncate((c.count * 0.5)), truncate((c.count * 0.75)));
        let e = c._inner[d];
        let f = c._inner[c.count - 1];
        let g = new FinancialDataPosition();
        g.symbol = f.symbol;
        g.company = f.company;
        g.data = c;
        g.tradeDate = e.date;
        g.shares = this.p.next2(100, 900);
        g.costPerShare = e.close;
        g.currentPrice = f.close;
        return g;
    }
    generateData(a, b = "") {
        let c = dateNow();
        this.f = dateFromValues(c.getFullYear(), dateGetMonth(c), c.getDate(), 16, 30, 0, 0);
        this.g = dateAddMonths(this.f, -this.dateRange);
        let d = Math.ceil(timeSpanTotalHours((+(this.f) - +(this.g))) / this.dateInterval);
        let e = dateAddHours(this.f, -d * this.dateInterval);
        if (this.g.getTime() > e.getTime()) {
            this.g = e;
        }
        let f = FinancialDataLookup.get(a);
        f.date = this.g;
        if (this.priceStart > 0) {
            f.open = this.priceStart;
        }
        if (this.volumeStart > 0) {
            f.volume = this.volumeStart;
        }
        this.volumeRange = f.volume * 0.1;
        this.priceRange = f.open * 0.05;
        this.priceMinimum = f.open - (f.open * 0.2);
        this.priceMaximum = f.open + (f.open * 0.2);
        f.high = f.open + (f.open * 0.02);
        f.low = f.open - (f.open * 0.02);
        f.close = f.open + ((this.p.nextDouble() - 0.5) * this.priceRange);
        let g = new FinancialDataSource();
        g.symbol = a;
        if (!stringIsNullOrEmpty(b)) {
            f.company = b;
            g.company = b;
        }
        else if (!stringIsNullOrEmpty(f.company)) {
            g.company = f.company;
        }
        else {
        }
        do {
            if (this.useAfterHours || !this.d(f.date)) {
                g.add(f);
            }
            f = this.generateItem(f);
        } while (f.date.getTime() < this.f.getTime());
        return g;
    }
    d(a) {
        if (a.getDay() == 6) {
            return true;
        }
        if (a.getDay() == 0) {
            return true;
        }
        let b = timeSpanInit1(9, 30, 0);
        let c = timeSpanInit1(16, 30, 0);
        let d = dateGetTimeOfDay(a);
        if (d < b || d > c) {
            return true;
        }
        return false;
    }
    generateItem(a) {
        let b = (this.p.nextDouble() - 0.49) * this.priceRange;
        let c = a.close;
        let d = a.close + b;
        if (d < this.priceMinimum) {
            d = this.priceMinimum + b;
        }
        if (d > this.priceMaximum) {
            d = this.priceMaximum - b;
        }
        let e = (this.p.nextDouble() * 1.5 * Math.abs(b));
        let f = (this.p.nextDouble() * 1.5 * Math.abs(b));
        if (d > c) {
            e = d + e;
            f = c - f;
        }
        else {
            e = c + e;
            f = d - f;
        }
        let g = (this.p.nextDouble() - 0.49) * this.volumeRange;
        let h = truncate(Math.abs(a.volume + g));
        let i = dateAddHours(a.date, this.dateInterval);
        if (i.getTime() > this.f.getTime()) {
            i = this.f;
        }
        let j = ((() => {
            let $ret = new FinancialDataItem(0);
            $ret.date = dateAddHours(a.date, this.dateInterval);
            $ret.symbol = a.symbol;
            $ret.company = a.company;
            $ret.close = round10N(d, 2);
            $ret.open = round10N(c, 2);
            $ret.high = round10N(e, 2);
            $ret.low = round10N(f, 2);
            $ret.volume = h;
            return $ret;
        })());
        return j;
    }
}
FinancialDataGenerator.$t = markType(FinancialDataGenerator, 'FinancialDataGenerator');
